#include <Arduino.h>

bool DEBUG = false;

// Definieer de pinnen voor de input LEDs
const int inputRedPin = 4;
const int inputGreenPin = 3;
const int inputBluePin = 2;

// Definieer de pinnen voor de output LEDs
const int outputRedPin = 5;
const int outputGreenPin = 6;
const int outputBluePin = 7;

// Variabelen voor het detecteren van de knippersnelheid van de blauwe input LED
volatile unsigned long lastInterruptTime = 0;
volatile unsigned long interval = 0;
volatile bool fastBlink = false;
volatile bool slowBlink = false;

int blinkInterval = 500;
unsigned long LastBlinkChange;
bool blinkState;

bool RFID_OK = false;

enum evseState
{
  ERROR,
  AVAILABLE,
  CONNECTED,
  CHARGING,
  WAITINGFORCHARGEING,
  OFF
};
evseState curstate = ERROR;

void resetBlink()
{
  fastBlink = false;
  slowBlink = false;
}

// Functie die wordt aangeroepen bij een interrupt
void handleBlueInterrupt()
{
  unsigned long currentTime = millis();
  interval = currentTime - lastInterruptTime;
  lastInterruptTime = currentTime;

  if (interval < 600)
  { // Als interval minder dan 250 ms is (2x per seconde)
    fastBlink = true;
    slowBlink = false;
  }
  else if (interval >= 600 && interval < 1200)
  { // Tussen 500 ms en 1 seconde (1x per seconde)
    slowBlink = true;
    fastBlink = false;
  }
  else
  {
    resetBlink();
  }
}

void setup()
{
  // Start de seriële communicatie
  if (DEBUG)
    Serial.begin(9600);
  // Stel de input pinnen in als ingangen
  pinMode(inputRedPin, INPUT_PULLUP);
  pinMode(inputGreenPin, INPUT_PULLUP);
  pinMode(inputBluePin, INPUT_PULLUP);

  // Stel de output pinnen in als uitgangen
  pinMode(outputRedPin, OUTPUT);
  pinMode(outputGreenPin, OUTPUT);
  pinMode(outputBluePin, OUTPUT);

  // Zet alle output LEDs uit
  digitalWrite(outputRedPin, LOW);
  digitalWrite(outputGreenPin, LOW);
  digitalWrite(outputBluePin, LOW);

  // Stel de interrupt in voor de blauwe input LED
  attachInterrupt(digitalPinToInterrupt(inputBluePin), handleBlueInterrupt, FALLING);
}

void updateoutput()
{
  // Controleer de prioriteiten en zet de output LEDs
  if (curstate == ERROR)
  {
    // Rode input-LED heeft prioriteit
    digitalWrite(outputRedPin, HIGH);
    digitalWrite(outputGreenPin, LOW);
    digitalWrite(outputBluePin, LOW);
  }
  else if (curstate == CHARGING)
  {
    // Groene input-LED heeft tweede prioriteit
    digitalWrite(outputRedPin, LOW);
    digitalWrite(outputGreenPin, LOW);
    digitalWrite(outputBluePin, HIGH);
  }
  else if (curstate == AVAILABLE)
  {
    // Blauwe input-LED knippert langzaam
    digitalWrite(outputRedPin, LOW);
    digitalWrite(outputGreenPin, HIGH);
    digitalWrite(outputBluePin, LOW);
  }
  else if (curstate == CONNECTED)
  {
    // Blauwe input-LED knippert snel
    if(blinkState){
      digitalWrite(outputRedPin, LOW);
      digitalWrite(outputGreenPin, LOW);
      digitalWrite(outputBluePin, LOW);
    }else{
      digitalWrite(outputRedPin, HIGH);
      digitalWrite(outputGreenPin, HIGH);
      digitalWrite(outputBluePin, LOW);
    }
  }
  else if (curstate == WAITINGFORCHARGEING)
  {
    // Blauwe input-LED brand
    digitalWrite(outputRedPin, HIGH);
    digitalWrite(outputGreenPin, HIGH);
    digitalWrite(outputBluePin, LOW);
  }
  else
  {
    // Zet alles uit als geen voorwaarden gelden
    digitalWrite(outputRedPin, LOW);
    digitalWrite(outputGreenPin, LOW);
    digitalWrite(outputBluePin, LOW);
  }
}

void determinState()
{
  // Lees de status van de input LEDs
  int inputRedState = digitalRead(inputRedPin);
  int inputGreenState = digitalRead(inputGreenPin);
  int inputBlueState = digitalRead(inputBluePin);

  if (inputRedState == LOW)
  {
    curstate = ERROR;
    resetBlink();
  }
  else if (inputGreenState == LOW)
  {
    RFID_OK = true;
    curstate = CHARGING;
    resetBlink();
  }
  else if ((fastBlink && RFID_OK) || ( inputBlueState == LOW && (lastInterruptTime + 1000) < millis()))
  {
    RFID_OK = true;
    curstate = WAITINGFORCHARGEING;
    resetBlink();
  }
  else if (fastBlink)
  {
    curstate = CONNECTED;
  }
  else if (slowBlink)
  {
    RFID_OK = false;
    curstate = AVAILABLE;
  }
  else if(curstate != CHARGING)//moet omdat groene led pulseerd tijdens charge
  {
    curstate = OFF;
    resetBlink();
  }
}

void loop()
{
  determinState();
  updateoutput();

  if(LastBlinkChange + blinkInterval < millis()){
    LastBlinkChange = millis();
    blinkState = !blinkState;
  }

  if (DEBUG)
  {
    // Seriële logging voor debugging
    Serial.print(curstate);
    Serial.print(", Input Red: ");
    Serial.print(digitalRead(inputRedPin) == LOW ? "ON" : "OFF");
    Serial.print(", Input Green: ");
    Serial.print(digitalRead(inputGreenPin) == LOW ? "ON" : "OFF");
    Serial.print(", Input Blue: ");
    Serial.print(digitalRead(inputBluePin) == LOW ? "ON" : "OFF");
    Serial.print(" | Output Red: ");
    Serial.print(digitalRead(outputRedPin) == HIGH ? "ON" : "OFF");
    Serial.print(", Output Green: ");
    Serial.print(digitalRead(outputGreenPin) == HIGH ? "ON" : "OFF");
    Serial.print(", Output Blue: ");
    Serial.println(digitalRead(outputBluePin) == HIGH ? "ON" : "OFF");
  }
  // Even wachten voor de volgende loop
  delay(100);
}
